package com.voxelmodpack.hdskins.server;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Sets;
import com.google.common.collect.ImmutableMap.Builder;
import com.google.gson.annotations.Expose;
import com.mojang.authlib.GameProfile;
import com.mojang.authlib.exceptions.AuthenticationException;
import com.mojang.util.UUIDTypeAdapter;
import com.voxelmodpack.hdskins.gui.Feature;
import com.voxelmodpack.hdskins.util.IndentedToStringStyle;
import com.voxelmodpack.hdskins.util.MoreHttpResponses;
import com.voxelmodpack.hdskins.util.NetClient;

import java.io.IOException;
import java.util.*;

@Deprecated
@ServerType("bethlehem")
public class BethlehemSkinServer implements SkinServer {

    private static final String SERVER_ID = "7853dfddc358333843ad55a2c7485c4aa0380a51";

    private static final Set<Feature> FEATURES = Sets.newHashSet(
            Feature.UPLOAD_USER_SKIN,
            Feature.DOWNLOAD_USER_SKIN,
            Feature.MODEL_VARIANTS,
            Feature.MODEL_TYPES,
            Feature.LINK_PROFILE
    );

    @Expose
    private final String address;

    private BethlehemSkinServer(String address) {
        this.address = address;
    }

    @Override
    public Set<Feature> getFeatures() {
        return FEATURES;
    }

    @Override
    public TexturePayload loadProfileData(GameProfile profile) throws IOException {
        try (MoreHttpResponses response = new NetClient("GET", getPath(profile)).send()) {
            return response.requireOk().json(TexturePayload.class, "Invalid texture payload");
        }
    }

    @Override
    public void performSkinUpload(SkinUpload upload) throws IOException, AuthenticationException {
        SkinServer.verifyServerConnection(upload.getSession(), SERVER_ID);

        NetClient client = new NetClient("POST", address);

        client.putHeaders(createHeaders(upload));

        if (upload.getImage() != null) {
            client.putFile(upload.getType().toString().toLowerCase(Locale.US), "image/png", upload.getImage());
        }

        try (MoreHttpResponses response = client.send()) {
            response.requireOk();
        }
    }

    protected Map<String, ?> createHeaders(SkinUpload upload) {
        Builder<String, Object> builder = ImmutableMap.<String, Object>builder()
                .put("accessToken", upload.getSession().getToken())
                .put("user", upload.getSession().getUsername())
                .put("uuid", UUIDTypeAdapter.fromUUID(upload.getSession().getProfile().getId()))
                .put("type", upload.getType().toString().toLowerCase(Locale.US));

        if (upload.getImage() == null) {
            builder.put("clear", "1");
        } else {
            builder.put("model", upload.getMetadata().getOrDefault("model", "default"));
        }

        return builder.build();
    }

    private String getPath(GameProfile profile) {
        return String.format("%s/profile/%s", address, UUIDTypeAdapter.fromUUID(profile.getId()));
    }

    @Override
    public String toString() {
        return new IndentedToStringStyle.Builder(this)
                .append("address", address)
                .build();
    }
}
